window.Gleam.initSearch([{"doc":"lustre_http_lib","title":"lustre_http_lib","content":"# lustre_http\n\n[![Package Version](https://img.shields.io/hexpm/v/lustre_http)](https://hex.pm/packages/lustre_http)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lustre_http/)\n\nMake HTTP requests from `lustre` via its `effect` interface.\nMakes life easy when such requests are initiated by the user.\n\n## Quick start\n\nAdd to your Gleam project:\n\n```sh\ngleam add lustre_http\n```\n\nView the documentation at &lt;https://hexdocs.pm/lustre_http&gt;, although only `get_as_text` is documented a.t.m.\n","url":"index.html"},{"doc":"lustre_http","title":"Expect","content":"pub opaque type Expect(msg)\n An expectation of what kind of response we&#39;re expecting to receive. This is\n how you an teach lustre_http to turn HTTP responses into messages your app\n can handle.\n\n For simple cases, you can use [`expect_text`](#expect_text) and\n [`expect_json`](#expect_json). These functions handle the response for you\n and you just need to provide a function to turn the text or JSON into a\n message.\n\n For more complex cases, you can use [`expect_text_response`](#expect_text_response)\n instead. This function lets you handle the\n [gleam_http Response](https://hexdocs.pm/gleam_http/gleam/http/response.html#Response)\n directly and is useful if you want to handle specific HTTP status codes or\n read the response headers.\n\n\nSynonyms:\nlustre_http.Expect\nlustre_http Expect","url":"lustre_http.html#Expect"},{"doc":"lustre_http","title":"HttpError","content":"pub type HttpError {\n  BadUrl(String)\n  InternalServerError(String)\n  JsonError(json.DecodeError)\n  NetworkError\n  NotFound\n  OtherError(Int, String)\n  Unauthorized\n}\n A HTTP request might fail in a few different ways: some of these are errors\n from the server (e.g. a 404 `NotFound` error) but others are ways your request\n can fail on the client. This type enumerates all of them.\n\nBadUrl(String)\n Both [`get`](#get) and [`post`](#post) let you pass the request URL as a\n string. This error is returned if you pass in something that isn&#39;t a valid\n URL.\n\nInternalServerError(String)\n The server returned a 500 Internal Server Error. The body of the response\n is included as a string.\n\nJsonError(json.DecodeError)\n When you use [`expect_json`](#expect_json) to decode an incoming response,\n this error is returned if the body is not valid JSON or the decoder fails.\n\nNetworkError\n If you try and make a request while the client is offline, this error is\n returned.\n\nNotFound\n The server returned a 404 Not Found error.\n\nOtherError(Int, String)\n Any other non-200 response from the server that is not 404, 401 or 500 will\n be returned as this error. The status code and body of the response are\n included.\n\nUnauthorized\n The server returned a 401 Unauthorized error.\n\nSynonyms:\nlustre_http.HttpError\nlustre_http HttpError","url":"lustre_http.html#HttpError"},{"doc":"lustre_http","title":"expect_anything","content":"pub fn expect_anything(\n  to_msg: fn(Result(Nil, HttpError)) -&gt; a,\n) -&gt; Expect(a)\n Expect any response. This is useful if you want to just fire off a request\n and make sure it was successful. If you want to handle the response body in\n some way, you should take a look at [`expect_text`](#expect_text) or\n [`expect_json`](#expect_json) instead.\n\nSynonyms:\nlustre_http.expect_anything\nlustre_http expect_anything","url":"lustre_http.html#expect_anything"},{"doc":"lustre_http","title":"expect_json","content":"pub fn expect_json(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  to_msg: fn(Result(a, HttpError)) -&gt; b,\n) -&gt; Expect(b)\n Expect a JSON response. The decoder is used to decode the JSON into a\n well-typed Gleam value. If this fails, the `JsonError` error variant will be\n returned.\n\n ### Usage\n\n \n import lustre_http as http\n import gleam/dynamic\n\n type Post {\n   Post(id: Int, title: String, body: String)\n }\n\n type Msg {\n   GotPosts(Result(List(Post), http.HttpError))\n }\n\n fn get_posts() -&gt; Effect(msg) {\n   let url = &quot;https://jsonplaceholder.typicode.com/posts&quot;\n   let decoder = dynamic.decode3(\n     Post,\n     dynamic.field(&quot;id&quot;, dynamic.int),\n     dynamic.field(&quot;title&quot;, dynamic.string),\n     dynamic.field(&quot;body&quot;, dynamic.string),\n   )\n\n   http.get(url, http.expect_json(dynamic.list(decoder)), GotPosts)\n }\n \n\nSynonyms:\nlustre_http.expect_json\nlustre_http expect_json","url":"lustre_http.html#expect_json"},{"doc":"lustre_http","title":"expect_text","content":"pub fn expect_text(\n  to_msg: fn(Result(String, HttpError)) -&gt; a,\n) -&gt; Expect(a)\n Expect a plain text response.\n\nSynonyms:\nlustre_http.expect_text\nlustre_http expect_text","url":"lustre_http.html#expect_text"},{"doc":"lustre_http","title":"expect_text_response","content":"pub fn expect_text_response(\n  on_response: fn(Response(String)) -&gt; Result(a, b),\n  on_failure: fn(HttpError) -&gt; b,\n  to_msg: fn(Result(a, b)) -&gt; c,\n) -&gt; Expect(c)\n Expect a [gleam_http `Response`](https://hexdocs.pm/gleam_http/gleam/http/response.html#Response)\n and handle it yourself. This is necessary if you want to handle specific\n HTTP status codes or read the response headers.\n\nSynonyms:\nlustre_http.expect_text_response\nlustre_http expect_text_response","url":"lustre_http.html#expect_text_response"},{"doc":"lustre_http","title":"get","content":"pub fn get(url: String, expect: Expect(a)) -&gt; Effect(a)\n Send a GET request to the given URL and say what kind of response you&#39;re\n expecting. If the url is invalid, the expect handler will receive a `BadUrl`\n error.\n\n ### Usage\n\n \n import lustre_http as http\n\n type Msg {\n   GotIpAddress(Result(String, http.HttpError))\n   WhoAmI\n }\n\n fn update(model, msg) {\n   case msg {\n     GotIpAddress(Ok(ip)) -&gt; ...\n     GotIpAddress(Error(err)) -&gt; ...\n     WhoAmI -&gt; #(\n       model,\n       http.get(&quot;https://api.ipify.org&quot;, http.expect_text(GotIpAddress)\n     )\n   }\n }\n \n\n If you need tighter control over the request - e.g. to set headers - you can\n construct the request manually using the [gleam_http](https://hexdocs.pm/gleam_http/gleam/http/request.html)\n package and then use [`send`](#send) instead.\n\nSynonyms:\nlustre_http.get\nlustre_http get","url":"lustre_http.html#get"},{"doc":"lustre_http","title":"post","content":"pub fn post(\n  url: String,\n  body: Json,\n  expect: Expect(a),\n) -&gt; Effect(a)\n Send a POST request to the given URL and say what kind of response you&#39;re\n expecting. If the url is invalid, the expect handler will receive a `BadUrl`\n error.\n\n ### Usage\n\n \n import lustre_http as http\n import gleam/json.{type Json}\n\n type Msg {\n   GotResponse(Result(Nil, http.HttpError))\n   CreatePost(body: Json)\n }\n\n fn update(model, msg) {\n   case msg {\n     GotResponse(Ok(_)) -&gt; ...\n     GotResponse(Error(err)) -&gt; ...\n     CreatePost(body) -&gt; #(\n       model,\n       http.post(\n         &quot;https://jsonplaceholder.typicode.com/posts&quot;,\n         body,\n         http.expect_anything(GotResponse))\n     )\n   }\n }\n \n\n If you need tighter control over the request - e.g. to set headers - you can\n construct the request manually using the [gleam_http](https://hexdocs.pm/gleam_http/gleam/http/request.html)\n package and then use [`send`](#send) instead.\n\nSynonyms:\nlustre_http.post\nlustre_http post","url":"lustre_http.html#post"},{"doc":"lustre_http","title":"send","content":"pub fn send(req: Request(String), expect: Expect(a)) -&gt; Effect(a)\n Send a [gleam_http `Request`](https://hexdocs.pm/gleam_http/gleam/http/request.html#Request)\n along with what kind of response you&#39;re expecting to receive. Once the request\n is complete, the response will be turned into a message you can handle in\n your `update` function.\n\n If you just want to make a simple GET or POST request, you might find either\n [`get`](#get) or [`post`](#post) easier to use!\n\nSynonyms:\nlustre_http.send\nlustre_http send","url":"lustre_http.html#send"},{"doc":"lustre_http","title":"lustre_http","content":"","url":"lustre_http.html"}]);